//****************************************************************************
//**
//**    SYS_MAIN.CPP
//**    System Main
//**
//****************************************************************************
//============================================================================
//    IMPLEMENTATION HEADERS
//============================================================================
#define SYS_DEFS_INCLUDE_WIN32 // for SYSW_Argc, SYSW_Argv,
                               // and SYSW_MessageBox
#include "sys_defs.h"
#include "sys_main.h"

//============================================================================
//    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
//============================================================================
#define SAFEMALLOC_MARKER 0x2b3d1234
#define SAFEMALLOC_USEMARKER // enable malloc/free marker verify

//============================================================================
//    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
//============================================================================
//============================================================================
//    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
//============================================================================
//============================================================================
//    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
//============================================================================
//============================================================================
//    IMPLEMENTATION PRIVATE DATA
//============================================================================
static float sys_baseTime;
static float sys_lastTime;

//============================================================================
//    INTERFACE DATA
//============================================================================
char sys_programPath[SYS_MAX_PATH];
float sys_curTime, sys_frameTime;

//============================================================================
//    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
//============================================================================
static void UpdateFrameTimes();
static void ResetFrameTimes();

//============================================================================
//    IMPLEMENTATION PRIVATE FUNCTIONS
//============================================================================
static void UpdateFrameTimes()
{
	sys_curTime = M_TimeFloat() - sys_baseTime;
	sys_frameTime = sys_curTime - sys_lastTime;
	sys_lastTime = sys_curTime;
}

static void ResetFrameTimes()
{
	sys_baseTime = M_TimeFloat();
	sys_lastTime = sys_curTime = sys_frameTime = 0.0;
}

//============================================================================
//    INTERFACE FUNCTIONS
//============================================================================

// The "Big Three" control functions

// initialization
void SYS_Init()
{
	if (!M_CmdLineOption("noalert", 0)) // example of command line option use
	{
		SYS_Alertf("This program does nothing but spawn a main window "
				   "and sit there,\nallocating and freeing some memory "
				   "over and over again.\n\n"
				   "Hit ESC when you want it to go away, or wait ten "
				   "seconds and it\nwill go away by itself.\n\n"
				   "BTW, you can run this program with the -noalert "
				   "option to skip this message.");
	}
	
	ResetFrameTimes(); // initialize frame times from this point
}

// main loop
void SYS_Frame()
{
	UpdateFrameTimes();

	// kill time by allocating a meg and freeing it
	byte *p = SYS_MALLOC(byte, 1024*1024);
	SYS_FREE(p);

	// has ten seconds passed?  If so, quit automatically
	if (sys_curTime >= 10.0)
		SYS_Quit();
}

// de-initialization
void SYS_Shutdown()
{
	// nothing required here yet
}

// alert notification
void SYS_Alertf(char* text, ...)
{
	static char textBuffer[1024];
	va_list args;
	va_start(args, text);
	vsprintf(textBuffer, text, args);
	va_end(args);
	SYSW_MessageBox(APPTITLE " Notification", MB_OK, textBuffer);
}

// exit without error
void SYS_Quit()
{
	SYS_Shutdown();
	exit(0);
}

// exit with fatal error
void SYS_Errorf(char *text, ...)
{
	static char textBuffer[1024];
	va_list args;
	va_start(args, text);
	vsprintf(textBuffer, text, args);
	va_end(args);
	SYSW_MessageBox("Fatal Error", MB_OK, textBuffer);
	SYS_Shutdown();
	exit(1);
}

// command line argument functions, call windows equivalents
byte SYS_Argc()
{
	return(SYSW_Argc());
}

char* SYS_Argv(byte index)
{
	return(SYSW_Argv(index));
}

// safe malloc and free wrappers, with optional marker verify (4 byte cost)
void* SYS_SafeMalloc(int size)
{
#ifdef SAFEMALLOC_USEMARKER	
	size += 4;
#endif
	void *res = malloc(size);
	if (!res)
		SYS_Errorf("SYS_SafeMalloc: Out of memory on %d bytes", size);
#ifdef SAFEMALLOC_USEMARKER	
	*((long *)res) = SAFEMALLOC_MARKER;
	return((long *)res+1);
#else
	return(res);
#endif
}

void SYS_SafeFree(void **ptr)
{
	long *p;
	if (!(*ptr))
		SYS_Errorf("SYS_SafeFree: Attempt to free null ptr");
	p = (long *)*ptr;
#ifdef SAFEMALLOC_USEMARKER	
	p -= 1;
	if (*p != SAFEMALLOC_MARKER)
		SYS_Errorf("SYS_SafeFree: Attempt to free non-SafeMalloc'ed pointer");
#endif
	free(p);
	*ptr = NULL;
}

//============================================================================
//    INTERFACE CLASS BODIES
//============================================================================

//****************************************************************************
//**
//**    END MODULE SYS_MAIN.CPP
//**
//****************************************************************************

